local Grid2IndicatorButton = Grid2:NewModule("Grid2IndicatorButton", "LibSink-2.0")

local DBL = LibStub:GetLibrary("LibDBLayers-1.0")


local function Button_Create(self, parent)
	local f = CreateFrame("Frame", nil, parent)
	f:SetBackdrop({
		edgeFile = "Interface\\Addons\\Grid2\\white16x16", edgeSize = 2,
		insets = {left = 2, right = 2, top = 2, bottom = 2},
	 })

	local Button = f:CreateTexture(nil, "ARTWORK")
	f.Button = Button
	Button:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	Button:SetAllPoints()

	local Cooldown = CreateFrame("Cooldown", nil, f, "CooldownFrameTemplate")
	f.Cooldown = Cooldown
	Cooldown:SetAllPoints(f)
	Cooldown:Hide()

	local Text = Cooldown:CreateFontString(nil, "OVERLAY")
	f.Text = Text
	Text:SetAllPoints()
	Text:SetFontObject(GameFontHighlightSmall)
	Text:SetFont(Text:GetFont(), self.dbx.fontSize)
	Text:SetJustifyH("CENTER")
	Text:SetJustifyV("CENTER")
	Text:Hide()

	parent[self.name] = f
end

local function Button_Layout(self, parent)
	local Button = parent[self.name]
	Button:ClearAllPoints()
	Button:SetFrameLevel(parent:GetFrameLevel() + self.frameLevel)
	Button:SetPoint(self.anchor, parent, self.anchorRel, self.offsetx, self.offsety)
	local ButtonSize = self.dbx.buttonSize
	Button:SetWidth(buttonSize)
	Button:SetHeight(buttonSize)
end

local function Button_GetBlinkFrame(self, parent)
	return parent[self.name]
end

local GetTime = GetTime
local function Button_OnUpdate(self, parent, unit, status)
	local Button = parent[self.name]
	if not status then
		Button:Hide()
		return
	end
	Button.Icon:SetTexture(status:GetIcon(unit))
	Button:Show()
	if status.GetTexCoord then
		Button.Icon:SetTexCoord(status:GetTexCoord(unit))
	else
		Button.Icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	end
	if (status.GetColor) then
		local r, g, b, a = status:GetColor(unit)

		if (status.GetBorder and status:GetBorder(unit) > 0) then
			Button:SetBackdropBorderColor(r, g, b, a)
		else
			Button:SetBackdropBorderColor(0, 0, 0, 0)
		end
		Button.Icon:SetAlpha(a or 1)
	else
		Button:SetBackdropBorderColor(1, 0, 0)
		Button.Icon:SetAlpha(1)
	end
	if status.GetCount then
		local count = status:GetCount(unit)
		if not count or count <= 1 then count = "" end
		Button.Text:SetText(count)
		Button.Text:Show()
	else
		Button.Text:Hide()
	end
	if (status.GetExpirationTime and status.GetDuration) then
		local expirationTime, duration = status:GetExpirationTime(unit), status:GetDuration(unit)
		if expirationTime and duration then
			Button.Cooldown:SetCooldown(expirationTime - duration, duration)
			Button.Cooldown:Show()
		else
			Button.Cooldown:Hide()
		end
	else
		Button.Cooldown:Hide()
	end
end

local function Button_SetButtonSize(self, parent, buttonSize)
	local Button = parent[self.name]
	Button:SetWidth(buttonSize)
	Button:SetHeight(buttonSize)
end

local function Button_Disable(self, parent)
	local Button = parent[self.name]
	Button:Hide()

	self.GetBlinkFrame = nil
	self.Layout = nil
	self.OnUpdate = nil
	self.SetButtonSize = nil
end

local function Button_UpdateDB(self, dbx)
	local oldType = self.dbx and self.dbx.type or dbx.type
	local location = Grid2.locations[dbx.location]

	self.frameLevel = dbx.level
	self.anchor = location.point
	self.anchorRel = location.relPoint
	self.offsetx = location.x
	self.offsety = location.y
	self.Create = Button_Create
	self.GetBlinkFrame = Button_GetBlinkFrame
	self.Layout = Button_Layout
	self.OnUpdate = Button_OnUpdate
	self.SetButtonSize = Button_SetButtonSize
	self.Disable = Button_Disable
	self.UpdateDB = Button_UpdateDB

	self.dbx = dbx
end


local function CreateButton(indicatorKey, dbx)
	local indicator = Grid2.indicatorPrototype:new(indicatorKey)

	Button_UpdateDB(indicator, dbx)

	Grid2:RegisterIndicator(indicator, { "color" })
	return indicator
end

Grid2.setupFunc["button"] = CreateButton



-- Hook the loading of setup so our objects can get initialized.
local prev_SetupCustom = Grid2.SetupCustom
function Grid2:SetupCustom(setup, objects, profileCurrentKey, ...)
	prev_SetupCustom(self, setup, objects, profileCurrentKey, ...)

	Grid2IndicatorButton.dbx = DBL:GetRuntimeGlobals(Grid2.dblData, "buttons")
	Grid2.Grid2IndicatorButton = Grid2IndicatorButton
	Grid2.buttons = {}
end

-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(dblData, ...)
	local upgrade = prev_LoadOptions(self, dblData, ...)

	upgrade = DBL:LoadOptions("Grid2IndicatorButtonOptions", dblData, nil, "account", 1, Grid2.dblData.classKey, 1, "globals", 1) or upgrade

	return upgrade
end

-- Hook UpgradeDefaults to blend in default options if current ones are old.
local prev_UpgradeDefaults = Grid2.UpgradeDefaults
function Grid2:UpgradeDefaults(dblData, ...)
	local flatten = prev_UpgradeDefaults(self, dblData, ...)

	local Grid2IndicatorButtonOptions = Grid2Options.plugins["Grid2IndicatorButtonOptions"]
	if (Grid2IndicatorButtonOptions) then
		flatten = DBL:UpgradeDefaults("Grid2IndicatorButtonOptions", dblData, Grid2IndicatorButtonOptions.UpgradeDefaults, "account", 1, Grid2.dblData.classKey, 1, "globals", 1) or flatten
	end
	
	return flatten
end
